import pygame
from pygame.locals import *

# A demo of the PixelArray + SRCCOLORKEY bug. Run it and follow the prompt in the title bar.
# Copyright Gummbum, 4/18/2017. Free software, no warranty express or implied.


def graduate(img, primary, secondary):
    pa = pygame.PixelArray(img)
    # Legacy: PixelArray.shape is not available in pygame 1.9.1
    # cols, rows = pa.shape
    cols, rows = img.get_size()
    step = 1.0 / rows
    pf = 1.0
    sf = 0.0
    pri3 = primary[:3]
    for y in range(rows):
        grad = [int(p * pf + s * sf) for p, s in zip(primary[:3], secondary[:3])]
        grad = Color(*grad)
        row = pa[:, y:y+1]
        # row.replace() does not preserve alpha...
        # row.replace(primary, grad)
        for x, ci in enumerate(row):
            try:
                # pygame 1.9.1 (and 2?)
                c = img.unmap_rgb(ci)
            except TypeError:
                # pygame 1.9.3
                c = img.unmap_rgb(ci[0])
            if c[:3] == pri3:
                grad[3] = c[3]
                row[x] = grad
        pf -= step
        sf += step
    img.unlock()
    return img


def render(s, antialias, primary, secondary):
    img = font.render(s, antialias, primary)
    graduate(img, primary, secondary)
    if antialias:
        pygame.display.set_caption('Antialias is True - Press SPACE to see the problem')
    else:
        pygame.display.set_caption('Antialias is False - Press SPACE to see it correctly')
    return img



pygame.init()
screen = pygame.display.set_mode((640, 480))
screen_rect = screen.get_rect()
clock = pygame.time.Clock()
max_fps = 30
dt = 1.0 / max_fps

primary = Color('red')
secondary = Color('yellow')
font = pygame.font.Font('VeraBd.ttf', 40)

antialias = True
the_text = 'GIMME FUN FX'
img = render(the_text, antialias, primary, secondary)

while True:
    clock.tick(max_fps)
    for e in pygame.event.get():
        if e.type == KEYDOWN:
            if e.key == K_SPACE:
                antialias = not antialias
                img = render(the_text, antialias, primary, secondary)
            if e.key == K_ESCAPE:
                quit()
        elif e.type == QUIT:
            quit()
    screen.fill((0, 0, 0))
    screen.blit(img, (10, 10))
    pygame.display.flip()
