#!/usr/bin/env python
# -*- coding: utf-8 -*-

# A demo to test font readability on a random color disk background.
# Copyright Gummbum, 4/18/2017. Free software, no warranty express or implied.

import math
import random

import pygame
from pygame.locals import Color, QUIT, KEYDOWN, K_ESCAPE

import hudlight


# Some decorative lines to break up the monotone background
def make_lines(lines, num_lines):
    origin = screen_rect.center
    radius = max(screen_rect.w // 2, screen_rect.h // 2)
    angle = 0.0
    step = 360.0 / num_lines
    while angle <= 360.0:
        radians_ = math.radians(angle)
        x = origin[0] + radius * math.cos(radians_)
        y = origin[1] + radius * math.sin(radians_)
        lines.append((origin, (x, y)))
        angle += step


def nudge_colors(color_n, last_which, last_sign, colors):
    color_n %= len(colors)
    color0 = colors[color_n]
    color_n = (color_n + 1) % len(colors)
    color1 = colors[color_n]
    ci = 0, 1, 2
    signs = -1, 1

    rc = random.choice
    rr = random.random
    which = rc(ci) if rr() * len(colors) < 4.0 else last_which
    sign = rc(signs) if rr() * len(colors) < 25.0 else last_sign
    c = color0[which]
    r = rr() * sign * 2.0
    c += r
    if c < 0.0 or c > 255.0:
        # TODO: when it hits a bound, it tends to cling; may want to bump this more towards the middle
        c -= r * 2
    color1[which] = c

    return color_n, which, sign


pygame.init()
clock = pygame.time.Clock()
screen = pygame.display.set_mode((320, 240))
screen_rect = screen.get_rect()
pygame.display.set_caption('HUD Demo 1 | Q or ESC to quit')
clear_color = Color('grey20')
progress_color = Color('midnightblue')

max_fps = 60

# Data used to populate the HUD.
color_n = 0
last_which = 2
last_sign = 1
translate_which = 'red green blue'.split()

# Make the HUD.
hud = hudlight.HUD(size=22, fgcolor='red', gcolor='yellow', scolor='black')
hud.add('fps', 'FPS {}', 0)
hud.add('progress', 'Progress {} / {}', color_n, 0)
hud.add('purple', 'Purple {0[0]:0.1f}, {0[1]:0.1f}, {0[2]:0.1f}', Color('purple'))
hud.add('color0', 'Color0 {0[0]:0.1f}, {0[1]:0.1f}, {0[2]:0.1f}', (0, 0, 0))
hud.add('last_which', 'Changing {}, Sign {:+d}', translate_which[last_which], last_sign)

# Make the "game" data.
decorative_lines = []
make_lines(decorative_lines, 5 * 360)
decorative_colors = [list(Color('purple')) for o in decorative_lines]
decorative_enums = list(range(len(decorative_colors)))

running = True

while running:
    clock.tick(max_fps)

    for e in pygame.event.get():
        if e.type == QUIT:
            running = False
        elif e.type == KEYDOWN:
            if e.key == K_ESCAPE:
                running = False

    # Update the "game" logic.
    color_n, last_which, last_sign = nudge_colors(color_n, last_which, last_sign, decorative_colors)

    # Update the HUD items.
    hud.update_item('fps', int(clock.get_fps()))
    hud.update_item('progress', color_n, len(decorative_lines))
    if color_n == 0:
        # print(color_n, decorative_colors[0])
        hud.update_item('color0', decorative_colors[0])
    hud.update_item('last_which', translate_which[last_which], last_sign)

    # Do all the game rendering and draw the HUD on top.
    screen.fill(clear_color)
    draw_aaline = pygame.draw.aaline
    for i, (p1, p2), c in zip(decorative_enums, decorative_lines, decorative_colors):
        if i == color_n:
            c = progress_color
        draw_aaline(screen, c, p1, p2)
    hud.draw(screen)
    pygame.display.flip()
